@@ -1361,12 +1361,23 @@ void ControllerWidget::on_MainDialog_SdlEvent(SDL_Event* event)
                 break;
             }
 
-            // update controller button state
+            // update controller button state,
+            // we have to loop over each possible direction,
+            // then ensure the opposite direction is disabled
+            // because an axis can only be at one direction
+            // at a given time
+            const int directions[] =
+            {
+                0, 1
+            };
             for (auto& button : this->buttonWidgetMappings)
             {
-                if (button.buttonWidget->HasInputData(inputType, sdlAxis, sdlAxisDirection))
+                for (int direction : directions)
                 {
-                    this->controllerImageWidget->SetButtonState(button.button, sdlAxisButtonPressed);
+                    if (button.buttonWidget->HasInputData(inputType, sdlAxis, direction))
+                    {
+                        this->controllerImageWidget->SetButtonState(button.button, (direction == sdlAxisDirection && sdlAxisButtonPressed));
+                    }
                 }
             }
 
