@@ -17,6 +17,12 @@
 
 namespace NetplayCommon
 {
+    struct NetplayServerData
+    {
+        bool Dispatcher;
+        QString Data;
+    };
+
     #define NETPLAYCOMMON_SESSION_REGEX "[a-zA-Z0-9 ]+"
     #define NETPLAYCOMMON_NICKNAME_REGEX "[a-zA-Z0-9]+"
     #define NETPLAYCOMMON_PASSWORD_REGEX "[a-zA-Z0-9,.\\/<>?;:[\\]{}\\-=_+`~!@#$%^&*()]+"
@@ -28,10 +34,16 @@ namespace NetplayCommon
     QList<QString> GetPluginNames(QString md5QString);
 
     // Adds servers from json to combobox
-    void AddServers(QComboBox* comboBox, QJsonDocument document);
+    void AddServers(QComboBox* comboBox, QJsonDocument document, bool dispatcher = false);
 
     // Restores previously selected server
     void RestoreSelectedServer(QComboBox* comboBox);
+
+    // Returns whether server from comboBox is dispatcher
+    bool IsServerDispatcher(QComboBox* comboBox, int index = -1);
+
+    // Returns server data from comboBox
+    QString GetServerData(QComboBox* comboBox, int index = -1);
 }
 
 #endif // NETPLAYCOMMON_HPP
